package ru.dbotthepony.mc.otm.item;

import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.ICapabilityProvider;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.event.entity.player.EntityItemPickupEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.drive.DrivePool;
import ru.dbotthepony.mc.otm.capability.drive.IMatteryDrive;
import ru.dbotthepony.mc.otm.capability.drive.MatteryDrive;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.UUID;

public class ItemPortableCondensationDrive extends Item {
	public ItemPortableCondensationDrive() {
		super(new Properties().stacksTo(1).tab(OverdriveThatMatters.CREATIVE_TAB));
	}

	@SubscribeEvent
	public static void onPickupEvent(EntityItemPickupEvent event) {
		for (var stack : event.getPlayer().getInventory().items) {
			if (stack.getItem() instanceof ItemPortableCondensationDrive) {
				var cap = stack.getCapability(MatteryCapability.DRIVE).resolve().get();

				var remaining = cap.insertItem(event.getItem().getItem(), false);

				if (remaining.getCount() == event.getItem().getItem().getCount()) {
					continue;
				}

				event.setCanceled(true);

				if (remaining.getCount() > 0) {
					event.getItem().getItem().setCount(remaining.getCount());
				} else {
					event.getItem().discard();
					return;
				}
			}
		}
	}

	private static class DriveCapability implements ICapabilityProvider {
		private UUID uuid;
		private final ItemStack stack;
		private final LazyOptional<IMatteryDrive> resolver = LazyOptional.of(() -> {
			return DrivePool.get(uuid, (tag) -> {
				var drive = new MatteryDrive(4000);
				drive.deserializeNBT(tag);
				return drive;
			}, () -> new MatteryDrive(4000));
		});

		DriveCapability(ItemStack stack) {
			this.stack = stack;
		}

		@Nonnull
		@Override
		public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
			if (cap == MatteryCapability.DRIVE) {
				if (uuid == null) {
					var get_value = stack.getOrCreateTag().getLongArray("uuid");

					if (get_value.length == 2) {
						uuid = new UUID(get_value[0], get_value[1]);
					} else {
						uuid = UUID.randomUUID();
						stack.getOrCreateTag().putLongArray("uuid", new long[] { uuid.getMostSignificantBits(), uuid.getLeastSignificantBits() });
					}
				}

				return resolver.cast();
			}

			return LazyOptional.empty();
		}
	}

	@Nullable
	@Override
	public ICapabilityProvider initCapabilities(ItemStack stack, @Nullable CompoundTag nbt) {
		return new DriveCapability(stack);
	}
}
